@@ -2,7 +2,7 @@ package InfluxDB::LineProtocol;
 use strict;
 use warnings;
 
-our $VERSION = '1.005';
+our $VERSION = '1.006';
 
 # ABSTRACT: Write and read InfluxDB LineProtocol
 
@@ -42,14 +42,14 @@ sub import {
 }
 
 sub data2line {
-    my ( $measurment, $values, $tags, $timestamp ) = @_;
+    my ( $measurement, $values, $tags, $timestamp ) = @_;
 
     if ( @_ == 1 ) {
         # no $fields, so assume we already got a line
-        return $measurment;
+        return $measurement;
     }
 
-    my $key = $measurment;
+    my $key = $measurement;
     $key =~ s/([, ])/\\$1/g;
 
     # $tags has to be a hashref, if it's not, we dont have tags, so it's the timestamp
@@ -82,8 +82,10 @@ sub data2line {
         }
     }
     else {
-        $timestamp = join( '', gettimeofday(), '000' );
-        $timestamp .= '0' if length($timestamp) < 19;
+        # Get time of day returns (seconds, microseconds)
+        # $timestamp needs to be nanoseconds
+        # it must also be a string to avoid conversion to sci notations
+        $timestamp = sprintf "%s%06d000", gettimeofday();
     }
 
     # If values is not a hashref, convert it into one
@@ -115,7 +117,6 @@ sub data2line {
     return sprintf( "%s %s %s", $key, $fields, $timestamp );
 }
 
-
 sub line2data {
     my $line = shift;
     chomp($line);
@@ -127,9 +128,9 @@ sub line2data {
     $line=~/^(.*?) (.*) (.*)$/;
     my ($key, $fields, $timestamp) = ( $1, $2, $3);
 
-    my ( $measurment, @taglist ) = split( /,/, $key );
-    $measurment =~ s/ESCAPEDSPACE/ /g;
-    $measurment =~ s/ESCAPEDCOMMA/,/g;
+    my ( $measurement, @taglist ) = split( /,/, $key );
+    $measurement =~ s/ESCAPEDSPACE/ /g;
+    $measurement =~ s/ESCAPEDCOMMA/,/g;
 
     my $tags;
     foreach my $tagset (@taglist) {
@@ -155,18 +156,18 @@ sub line2data {
         $values->{$k} = $v;
     }
 
-    return ( $measurment, $values, $tags, $timestamp );
+    return ( $measurement, $values, $tags, $timestamp );
 }
 
 sub _data2line_0_9_2 {
-    my ( $measurment, $values, $tags, $timestamp ) = @_;
+    my ( $measurement, $values, $tags, $timestamp ) = @_;
 
     if ( @_ == 1 ) {
         # no $fields, so assume we already got a line
-        return $measurment;
+        return $measurement;
     }
 
-    my $key = $measurment;
+    my $key = $measurement;
     $key =~ s/([, ])/\\$1/g;
 
     # $tags has to be a hashref, if it's not, we dont have tags, so it's the timestamp
@@ -178,6 +179,7 @@ sub _data2line_0_9_2 {
                 # TODO check if sorting algorithm matches
                 #      http://golang.org/pkg/bytes/#Compare
                 my $v = $tags->{$k};
+                next unless defined $v;
                 $k =~ s/([, ])/\\$1/g;
                 $v =~ s/([, ])/\\$1/g;
                 push( @tags, $k . '=' . $v );
@@ -240,9 +242,9 @@ sub _line2data_0_9_2 {
     $line=~/^(.*?) (.*) (.*)$/;
     my ($key, $fields, $timestamp) = ( $1, $2, $3);
 
-    my ( $measurment, @taglist ) = split( /,/, $key );
-    $measurment =~ s/ESCAPEDSPACE/ /g;
-    $measurment =~ s/ESCAPEDCOMMA/,/g;
+    my ( $measurement, @taglist ) = split( /,/, $key );
+    $measurement =~ s/ESCAPEDSPACE/ /g;
+    $measurement =~ s/ESCAPEDCOMMA/,/g;
 
     my $tags;
     foreach my $tagset (@taglist) {
@@ -267,7 +269,7 @@ sub _line2data_0_9_2 {
         $values->{$k} = $v;
     }
 
-    return ( $measurment, $values, $tags, $timestamp );
+    return ( $measurement, $values, $tags, $timestamp );
 }
 
 1;
@@ -284,7 +286,7 @@ InfluxDB::LineProtocol - Write and read InfluxDB LineProtocol
 
 =head1 VERSION
 
-version 1.005
+version 1.006
 
 =head1 SYNOPSIS
 
@@ -330,7 +332,7 @@ and C<0.9.2> if you ask nicely.
 C<data2line> takes various parameters and converts them to an
 InfluxDB Line.
 
-C<metric> has to be valid InfluxDB measurment name. Required.
+C<metric> has to be valid InfluxDB measurement name. Required.
 
 C<value> can be either a scalar, which will be turned into
 "value=$value"; or a hashref, if you want to write several values (or
